/***************************************************************************
                          kcpufreq.cpp  -  description
                             -------------------
    begin                : Mi Okt 27 20:10:17 CEST 2004
    copyright            : (C) 2004 by Sebastian Schaffert
    email                : wastl@wastl.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <qfile.h>
#include <qstring.h>


#include "kcpufreq.h"

#include <unistd.h>


#ifdef HAVE_CPUFREQ_H
#ifdef HAVE_LIBCPUFREQ
#define USE_LIB_CPUFREQ 1
#endif
#endif

#ifdef USE_LIB_CPUFREQ
extern "C" 
{
#include <cpufreq.h>
}
#endif

extern "C"
{
  KPanelApplet* init( QWidget *parent, const QString configFile)
  {
    KGlobal::locale()->insertCatalogue("kcpufreq");
    return new Kcpufreq(configFile, KPanelApplet::Normal,
			KPanelApplet::About,
			parent, "kcpufreq");
  }
}

Kcpufreq::Kcpufreq(const QString& configFile, Type type, int actions, QWidget *parent, const char *name) 
  : KPanelApplet(configFile, type, actions, parent, name)
{
  // Get the current application configuration handle
  ksConfig = config();

  // initialise pixmaps
  pixmaps[0]=KGlobal::iconLoader()->loadIcon(PIXMAP_NA,KIcon::Panel);
  pixmaps[1]=KGlobal::iconLoader()->loadIcon(PIXMAP_25,KIcon::Panel);
  pixmaps[2]=KGlobal::iconLoader()->loadIcon(PIXMAP_50,KIcon::Panel);
  pixmaps[3]=KGlobal::iconLoader()->loadIcon(PIXMAP_75,KIcon::Panel);
  pixmaps[4]=KGlobal::iconLoader()->loadIcon(PIXMAP_100,KIcon::Panel);


  mainView = new myview(this);   
  mainView->show();
  
  this->previous_freq_cur = 0;
  this->previous_freq_max = 0;

  this->startTimer(500);
}

Kcpufreq::~Kcpufreq()
{
}

void Kcpufreq::about()
{
  KMessageBox::about(0, i18n("Displays CPU frequency on CPUs with variable speed\n(c)2004 by Sebastian Schaffert\nwastl@wastl.net"));
}

/*
  void Kcpufreq::help()
  {
  KMessageBox::information(0, i18n("This is a help box"));
  }

  void Kcpufreq::preferences()
  {
  KMessageBox::information(0, i18n("This is a preferences box"));  
  }
*/


int Kcpufreq::widthForHeight( int height ) const
{
  return 26 + (height * 2);
}

int Kcpufreq::heightForWidth( int width ) const
{
  return (width-26) / 2;
}

/**
 * if the size of the applet changes, resize the widget appropriately
 */
void Kcpufreq::resizeEvent( QResizeEvent *)
{
  mainView->freqLabel->resize(this->width(),this->height());  
}


#ifndef USE_LIBCPU_FREQ
#define CPUFreq_MAX 1
#define CPUFreq_CUR 2


/**
 * read current/maximal frequency from the respective files in /sys
 * (see definition of FREQ_CUR and FREQ_MAX in kcpufreq.h)
 */
unsigned long get_cpufreq(int what)
{
  QFile ff(what&CPUFreq_CUR?FREQ_CUR:FREQ_MAX);
  if(ff.exists() && ff.open(IO_ReadOnly)) {
    QString s;
    ff.readLine(s,20);
    return s.toLong();
  } else {
    return 0;
  }
}
#endif


/**
 * React on QTimerEvents that occur every 500ms (see constructor above).
 *
 * Updates the applet if the frequency has changed.
 */
void Kcpufreq::timerEvent(QTimerEvent *)
{
  int ratio;
  // read in current frequency
#ifndef USE_LIB_CPUFREQ
  unsigned long freq_cur = get_cpufreq(CPUFreq_CUR);
  unsigned long freq_max = get_cpufreq(CPUFreq_MAX);
#else
  unsigned long freq_cur = cpufreq_get_freq_kernel(0);
  unsigned long freq_max, freq_min;
  cpufreq_get_hardware_limits(0,&freq_min,&freq_max);
#endif

  ratio  = 100*freq_cur/freq_max;

  // if the frequency changed since last check, update the icon and
  // text of the myView widget appropriately, otherwise do nothing
  if(freq_cur != this->previous_freq_cur || 
     freq_max != this->previous_freq_max) {
    QString qs;
    if(freq_cur >= 1000000) {
      qs = QString::number((double)freq_cur / 1000000,'f',2) + " GHz";
    } else if(freq_cur > 0) {
      qs = QString::number(freq_cur / 1000) + " MHz";
    } else {
      qs = "n/a";
    }
	
    mainView->freqLabel->setText(qs);
    mainView->freqIcon->setPixmap(pixmaps[ratio/25]);
    mainView->update();

    // save new frequency so that we can detect changes
    this->previous_freq_cur = freq_cur;
    this->previous_freq_max = freq_max;
  }
}

