/***************************************************************************
                          kcpufreq.h  -  description
                             -------------------
    begin                : Mi Okt 27 20:10:17 CEST 2004
    copyright            : (C) 2004 by Sebastian Schaffert
    email                : wastl@wastl.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KCPUFREQ_H
#define KCPUFREQ_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kpanelapplet.h>
#include <qstring.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <kconfig.h>
#include "myview.h"

#define FREQ_CUR "/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq"
#define FREQ_MAX "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq"

#define PIXMAP_NA "cpufreq_na"
#define PIXMAP_25 "cpufreq_25"
#define PIXMAP_50 "cpufreq_50"
#define PIXMAP_75 "cpufreq_75"
#define PIXMAP_100 "cpufreq_100"


class Kcpufreq : public KPanelApplet
{
  Q_OBJECT

public:
  /**
   * Construct a @ref KPanelApplet just like any other widget.
   *
   * @param configFile The configFile handed over in the factory function.
   * @param Type The applet @ref type().
   * @param actions Standard RMB menu actions supported by the applet (see @ref action() ).
   * @param parent The pointer to the parent widget handed over in the factory function.
   * @param name A Qt object name for your applet.
   **/
  Kcpufreq(const QString& configFile, Type t = Normal, int actions = 0,
	   QWidget *parent = 0, const char *name = 0);

  /** destructor */
  ~Kcpufreq();

  /**
   * Retrieve a suggested width for a given height.
   *
   * Every applet should reimplement this function.
   *
   * Depending on the panel orientation the height (horizontal panel) or the
   * width (vertical panel) of the applets is fixed.
   * The exact values of the fixed size component depend on the panel size.
   *
   * On a horizontal panel the applet height is fixed, the panel will
   * call @ref widthForHeight(int height) with @p height
   * equal to 'the fixed applet height'
   * when laying out the applets.
   *
   * The applet can now choose the other size component (width)
   * based on the given height.
   *
   * The width you return is granted.
   **/
  int widthForHeight( int height ) const;


  /**
   * @return A suggested height for a given width.
   *
   * Every applet should reimplement this function.
   *
   * Depending on the panel orientation the height (horizontal panel) or the
   * width (vertical panel) of the applets is fixed.
   * The exact values of the fixed size component depend on the panel size.
   *
   * On a vertical panel the applet width is fixed, the panel will
   * call @ref heightForWidth(int width) with @p width
   * equal to 'the fixed applet width'
   * when laying out the applets.
   *
   * The applet can now choose the other size component (height)
   * based on the given width.
   *
   * The height you return is granted.
   **/
  int heightForWidth( int width) const;

  /**
   * Is called when the user selects "About" from the applets RMB menu.
   * Reimplement this function to launch a about dialog.
   *
   * Note that this is called only when your applet supports the About action.
   * See @ref Action and @ref KPanelApplet().
   **/
  void about();


protected:

  void resizeEvent( QResizeEvent* );

  /**
   * The kcpufreq applet uses the timer to update the display every 500 ms.
   *
   * The QTimerEvent parameter is unused.
   */
  void timerEvent(QTimerEvent* );


private:
  KConfig *ksConfig;

  // the main view component; its name is still from kdevelop, but unimportant anyway:-)
  myview *mainView;
  
  // stores the five different pixmaps indicating the CPU frequency
  QPixmap pixmaps[5];

  // used to detect changes in the frequency.
  unsigned long previous_freq_cur, previous_freq_max;

};

#endif

// Local Variables:
// mode: c++
// End:
